@@ -59,15 +59,33 @@
 
 static Lum::AppInfo info;
 
+typedef Lum::Model::StdRefTable<Hiscore::Entry,std::vector<Hiscore::Entry> > HiscoreEntriesModel;
+typedef Lum::Base::Reference<HiscoreEntriesModel>                            HiscoreEntriesModelRef;
+
+class HiscoreEntryDataProvider : public HiscoreEntriesModel::DataProvider
+{
+public:
+  std::wstring GetString(const HiscoreEntriesModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return iter->name;
+    case 2:
+      return Lum::Base::NumberToWString((unsigned long)iter->time);
+    default:
+      assert(false);
+    }
+  }
+};
+
 class ScoreView : public Lum::Dialog
 {
 private:
-  Lum::Model::ListTableRef list;
+  HiscoreEntriesModelRef list;
 
 public:
   ScoreView(Sweeper* sweeper)
-  : Dialog(),
-    list(new Lum::Model::ListTable(2))
+  : Dialog()
   {
     Hiscore *score;
 
@@ -75,25 +93,16 @@ public:
                     sweeper->GetAreaHeight(),
                     sweeper->GetMines());
 
-    for (size_t x=0; x<score->list.size(); x++) {
-      Lum::Model::ListTable::StdEntry* entry;
-
-      entry=new Lum::Model::ListTable::StdEntry(list);
-      entry->SetString(1,score->list[x].name);
-      entry->SetString(2,Lum::Base::NumberToWString((unsigned long)score->list[x].time));
-      list->Append(entry);
-    }
+    list=new HiscoreEntriesModel(score->list,new HiscoreEntryDataProvider());
   }
 
   void PreInit()
   {
-    Lum::ButtonRow        *bRow;
     Lum::Panel            *panel;
     Lum::Table            *table;
     Lum::Model::HeaderRef headerModel;
 
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
+    panel=Lum::VPanel::Create(true,true);
 
     headerModel=new Lum::Model::HeaderImpl();
     headerModel->AddColumn(L"Name",Lum::Base::Size::stdCharWidth,18,true);
@@ -109,11 +118,8 @@ public:
     table->SetHeaderModel(headerModel);
     panel->Add(table);
 
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateClose(GetClosedAction());
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateClose(GetClosedAction(),true,false));
 
     SetMain(panel);
 
@@ -148,14 +154,11 @@ public:
 
   void PreInit()
   {
-    Lum::ButtonRow *bRow;
     Lum::Grid      *grid;
     Lum::Panel     *panel;
-    Lum::String    *string;
     Lum::Text      *text;
 
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
+    panel=Lum::VPanel::Create(true,true);
 
     grid=new Lum::Grid();
     grid->SetSpace(true,true);
@@ -165,12 +168,7 @@ public:
         text=new Lum::Text();
         text->SetText(Lum::Base::NumberToWString((unsigned long)x+1)+L":");
         grid->SetObject(0,x,text);
-
-        string=new Lum::String();
-        string->SetFlex(true,false);
-        string->SetWidth(Lum::Base::Size::stdCharWidth,10);
-        string->SetModel(name);
-        grid->SetObject(1,x,string);
+        grid->SetObject(1,x,Lum::String::Create(name,10,true,false));
 
         text=new Lum::Text();
         text->SetAlignment(Lum::Text::left);
@@ -198,12 +196,8 @@ public:
     }
 
     panel->Add(grid);
-
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateClose(closeAction);
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateClose(closeAction,true,false));
 
     SetMain(panel);
 
@@ -225,18 +219,37 @@ public:
   }
 };
 
+typedef Lum::Model::StdRefTable<Theme,std::vector<Theme> > ThemesModel;
+typedef Lum::Base::Reference<ThemesModel>                  ThemesModelRef;
+
+class ThemeDataProvider : public ThemesModel::DataProvider
+{
+public:
+  std::wstring GetString(const ThemesModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return iter->name;
+    case 2:
+      return Lum::Base::NumberToWString(iter->boxSize);
+    default:
+      assert(false);
+    }
+  }
+};
+
 class SelectTheme : public Lum::Dialog
 {
 private:
   Lum::Model::ActionRef              okAction;
-  Lum::Model::ListTableRef           list;
+  ThemesModelRef                     list;
   Lum::Model::SingleLineSelectionRef selection;
   std::wstring                       result;
 
 public:
   SelectTheme()
   : okAction(new Lum::Model::Action),
-    list(new Lum::Model::ListTable(2)),
+    list(new ThemesModel(themes,new ThemeDataProvider())),
     selection(new Lum::Model::SingleLineSelection),
     result(L"")
   {
@@ -244,28 +257,15 @@ public:
 
     Observe(okAction);
     Observe(selection);
-
-    for (size_t i=0; i<themes.size(); ++i) {
-      Lum::Model::ListTable::StdEntry *entry;
-
-      entry=new Lum::Model::ListTable::StdEntry(list);
-      entry->SetString(1,themes[i].name);
-      entry->SetString(2,Lum::Base::NumberToWString(themes[i].boxSize));
-
-      list->Append(entry);
-    }
   }
 
   void PreInit()
   {
-    Lum::ButtonRow        *bRow;
     Lum::Panel            *panel;
     Lum::Table            *table;
     Lum::Model::HeaderRef headerModel;
 
-
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
+    panel=Lum::VPanel::Create(true,true);
     panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*80)/100);
 
@@ -276,17 +276,15 @@ public:
     table=new Lum::Table();
     table->SetFlex(true,true);
     table->SetShowHeader(true);
+    table->GetTableView()->SetAutoHSize(true);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
     table->SetDoubleClickAction(okAction);
     panel->Add(table);
 
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction());
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction(),true,false));
 
     SetMain(panel);
 
@@ -362,18 +360,15 @@ public:
 
   void PreInit()
   {
-    Lum::Boolean     *boolean;
-    Lum::Button      *button;
-    Lum::ButtonRow   *bRow;
     Lum::Label       *label;
     Lum::Panel       *panel,*hPanel;
     Lum::Array       *array;
     Lum::Slider      *slider;
     Lum::TextValue   *textValue;
 
-    panel=new Lum::VPanel();
+    panel=Lum::VPanel::Create();
 
-    label=new Lum::Label();
+    label=Lum::Label::Create();
 
     slider=new Lum::HSlider();
     slider->SetFlex(true,false);
@@ -398,54 +393,26 @@ public:
     array->SetSpace(true,false);
     array->SetVerticalCount(1);
 
-    button=new Lum::Button();
-    button->SetFlex(true,true);
-    button->SetModel(smallAction);
-    button->SetText(L"_Small");
-    array->Add(button);
-
-    button=new Lum::Button();
-    button->SetFlex(true,true);
-    button->SetModel(mediumAction);
-    button->SetText(L"_Medium");
-    array->Add(button);
-
-    button=new Lum::Button();
-    button->SetFlex(true,true);
-    button->SetModel(bigAction);
-    button->SetText(L"_Big");
-    array->Add(button);
+    array->Add(Lum::Button::Create(L"_Small",smallAction,true));
+    array->Add(Lum::Button::Create(L"_Medium",mediumAction,true));
+    array->Add(Lum::Button::Create(L"_Big",bigAction,true));
 
     label->AddLabel(L"Presets:",array);
+    label->AddLabel(L"One mouse button mode:",Lum::Boolean::Create(oneButtonMode));
 
-    boolean=new Lum::Boolean();
-    boolean->SetModel(oneButtonMode);
-
-    label->AddLabel(L"One mouse button mode:",boolean);
-
-    hPanel=new Lum::HPanel();
-    hPanel->SetFlex(true,false);
+    hPanel=Lum::HPanel::Create(true,false);
 
     textValue=new Lum::TextValue();
     textValue->SetFlex(true,false);
     textValue->SetModel(themeName);
     hPanel->Add(textValue);
 
-    button=new Lum::Button();
-    button->SetFlex(false,true);
-    button->SetModel(themeAction);
-    button->SetText(L"...");
-    hPanel->Add(button);
+    hPanel->Add(Lum::Button::Create(L"...",themeAction,false,true));
 
     label->AddLabel(L"Theme:",hPanel);
-
     panel->Add(label);
-
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateClose(GetClosedAction());
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateClose(GetClosedAction(),true,false));
 
     SetMain(panel);
 
@@ -529,24 +496,16 @@ public:
 
   void PreInit()
   {
-    Lum::Button      *button;
     Lum::Panel       *vPanel,*hPanel;
-    Lum::View        *view;
     size_t           digitWidth,widestDigit;
     std::wstring     timeFormat,marksFormat;
     Lum::OS::FontRef font;
 
-    vPanel=new Lum::VPanel;
-    vPanel->SetFlex(true,true);
+    vPanel=Lum::VPanel::Create(true,true);
 
-    hPanel=new Lum::HPanel();
-    hPanel->SetFlex(true,false);
-
-    button=new Lum::Button();
-    button->SetText(L"_New game");
-    button->SetModel(newGameAction);
-    hPanel->Add(button);
+    hPanel=Lum::HPanel::Create(true,false);
 
+    hPanel->Add(Lum::Button::Create(L"_New game",newGameAction));
     hPanel->Add(new Lum::HSpace(Lum::Space::sizeNormal,true));
 
     font=Lum::OS::display->GetFont(Lum::OS::Display::fontTypeFixed,210);
@@ -594,19 +553,14 @@ public:
     hPanel->Add(time);
 
     vPanel->Add(hPanel);
-
-    vPanel->Add(new Lum::VSpace());
-
-    view=new Lum::View();
-    view->SetFlex(true,true);
+    vPanel->AddSpace();
 
     sweeper=new Sweeper();
     sweeper->SetFlex(true,true);
     Observe(sweeper->GetStatusModel());
     Observe(sweeper->GetMarksModel());
 
-    view->SetObject(sweeper);
-    vPanel->Add(view);
+    vPanel->Add(Lum::View::Create(sweeper,true,true));
 
     SetMain(vPanel);
 
